// image decoder include file
// v 0.1, 2002-07-18
// (C) Shinhei Hara, 2002
// All rights reserved.

#include <PalmOS.h>
#include <PalmTypes.h>
#include <Extensions/ExpansionMgr/VFSMgr.h>

// header version (last modified date)
#define ILHeaderVersion 0x020030409L

// code resource IDs
#define ILInitCodeID  0x03e8
#define ILLoadCodeID  0x07d0
#define ILInfoCodeID  0x0bb8

// About form and objects IDs
#define ILAboutFormID 0x0fa0
// About form should have OK btn only.

struct ImageInfoType {
  Coord width, height;
  UInt16 depth;
  UInt32 colors;
  char *comment;
};
typedef struct ImageInfoType ImageInfoType;
typedef ImageInfoType * ImageInfoPtr;

struct LoaderInitDataType {
  UInt32 version;          // ->   ILHeaderVersion that VFSi(appl) was compiled with
  UInt16 attr;             // <->  loader info attributes
  char extension[8];       // <-   target file type extension (ex. jpg, xpm)
  char label[32];          // <-   target file type description (ex. "XPM X11 Pixmap file" ..)
  DmResID resID;           // <-   target file icon bitmap resource if you have (to be copied)
};
typedef struct LoaderInitDataType LoaderInitDataType;
typedef LoaderInitDataType * LoaderInitDataPtr;
// Attribute Masks from plugin
#define LIAttrHasIcon   0x0001
// Attribute Masks from VFSi
#define LIAttrCPUIsARM  0x0100

struct IOFPointerDataType {
  char (*ioGetByteP)(FileRef);                           // read 1 byte from file
  UInt32 (*ioGetintP)(FileRef);                          // read 4 bytes and convert to big endian
  UInt16 (*ioGetshortP)(FileRef);                        // read 2 bytes and convert to big endian
};
typedef struct IOFPointerDataType IOFPointerDataType;

struct DecodeImageInfoType {
  FileRef f;
  ImageInfoPtr iiP;
  IOFPointerDataType iofp;
};
typedef struct DecodeImageInfoType DecodeImageInfoType;
typedef DecodeImageInfoType * DecodeImageInfoPtr;

// Utility function pointers -- Warnig!! Don't change the order of function pointers!
struct BSFPointerDataType {
  void (*BSFSetCtblP)(UInt32, UInt16 *, UInt8 *);        // set the palette for the image (for 4/8bits images)
  Err (*BSFSetDimensionsP)(Coord, Coord, UInt16, Boolean);  
                                                         // set the dims and setup the storage for the image
  void (*BSFStoreStartP)(void);                          // setup the writing the image pixels
  void (*BSFRevStoreStartP)(void);                       // setup the writing the image pixels in reverse order
  void (*BSFLineWriteP)(MemPtr);                         // write the line of the image into the image storage
  void (*BSFRevLineWriteP)(MemPtr);                      // write the line of the image into the image storage in rev
  void (*BSFRevChunkWriteP)(MemPtr, UInt32);             // write chunk of the image storage (reverse)
  void (*BSFChunkWriteP)(MemPtr, UInt32);                // write chunk of the image storage
  void (*BSFBlockWriteP)(MemPtr, Coord);                 // write image block into the image storage
  void (*BSFLineSkipP)(UInt32);                          // skip writing lines of the image
  void (*BSFRevLineSkipP)(UInt32);                       // skip writing lines of the image in reverse order
  void (*BSFStoreFinishP)(void);                         // finish writing the lines
  void (*BSFGetChunkSizeP)(UInt32 *, UInt32 *);          // Get the size of allocated chunks 
  char (*ioGetByteP)(FileRef);                           // read 1 byte from file
  UInt32 (*ioGetintP)(FileRef);                          // read 4 bytes and convert to big endian
  UInt16 (*ioGetshortP)(FileRef);                        // read 2 bytes and convert to big endian
  void (*showAlertP)(char *);                            // show message box
  // all of above are on 2002-07-02

};
typedef struct BSFPointerDataType BSFPointerDataType;

struct DecodeImageDataType {
  FileRef f;                   // -> target file reference
  UInt16 attr;                 // -> system attributes
  BSFPointerDataType bsfp;     // -> BSF API Pointers
};
typedef struct DecodeImageDataType DecodeImageDataType;
typedef DecodeImageDataType * DecodeImageDataPtr;
#define DICPUIsARM 0x0001
