// plugin include file
// v 0.1, 2002-07-18
// (C) Shinhei Hara, 2002
// All rights reserved.

#include <PalmOS.h>
#include <PalmTypes.h>
#include <Extensions/ExpansionMgr/VFSMgr.h>

// Header version
#define PMHeaderVersion 0x20040330L

// code resource IDs
#define PMInitCodeID  0x03e8
#define PMMainCodeID  0x07d0
#define PMInfoCodeID  0x0bb8

// About form ID
#define PMAboutFormID 0x0fa0
// About form should have one btn only.

// Info form ID
#define PMInfoFormID 0x0bb8

// Available upper and lower form IDs and entryFormID
#define PMFormHiID    0x0bb7
#define PMFormLowID   0x07d0

// Return Form
#define PMFileListFormID 9966

struct PMPluginFileType {  // define available file types in the class (following struct)
  UInt16 attr;             // <-   type info attributes
  char extension[8];       // <-   target file type extension (ex. txt, wav)
  char label[32];          // <-   target file type description (ex. "Plain text file" ..)
  DmResID resID;           // <-   target file icon bitmap resource (to be copied)
};
typedef struct PMPluginFileType PMPluginFileType;
typedef PMPluginFileType * PMPluginFileTypePtr;
// type info attrubute masks
//#define PMTAttrIconIsHR          0x0004
//#define PMTAttrIconIsV3Bmp       0x0010  // for PalmOS 5
#define PMTAttrCantHaveLink      0x0020  // if having a bookmark is not appropriate, this should be set.

struct PMPluginInitDataType {
  UInt32 version;                 // ->   PMHeaderVersion that VFSi(appl) was compiled with
  UInt16 attr;                    // <->  plugin attributes
  char menuText[32];              // <-   menu text you want to show in file browse form (you can have only one)
  PMPluginFileType ftype;         // <-   type data  
  MemHandle preserveH;            // <-   preserved memory chunk while app is running. (if no need, set 0)
};
typedef struct PMPluginInitDataType PMPluginInitDataType;
typedef PMPluginInitDataType * PMPluginInitDataPtr;
// from plugin to VFSi (low-byte)
#define PMIAttrHasMenu           0x0001  // plugin requires menu item
#define PMIAttrHasInfoForm       0x0002  // if file info form exist, check this mask
#define PMIAttrCanHandleNotify   0x0004  // if plugin can handle disp change notify
// from VFSi to plugin (Hi-byte)
#define PMIAttrMachineHasHRLib   0x0100
#define PMIAttrMachineIsOS5      0x0200  // for PalmOS 5
#define PMIAttrCPUIsARM          0x0400  // indicates that machine's cpu is arm family (or x86)
#define PMIAttrPINMgr            0x0800  // indicates that machine has PIN Manager

struct PMPluginExchangeDataType {
  UInt16 attr;                    //  <-> attributes for launch and quit situation 
  UInt16 volRefNum;               //  -> current volume reference
  char *filename;                 //  -> target filename                                 (Don't be freed)
  char *workingDir;               //  -> current working directory                       (Don't be freed)
  MemHandle checkedFilesH;        //  -> checked file basenames (NULL separated string)  (Don't be freed)
  Int16 checkedFilesNum;          //  -> the number of checked files
  DmOpenRef dbRef;                //  -> database ref for temporary storage              (Don't be closed)
  EventPtr evp;                   //  -> event ptr (be set not when launch - menu/open) 
  MemHandle preserveH;            //  <- preserved memory chunk while app is running.    (Don't be freed)
  MemPtr savedP;                  //  <- preserved pointer.   (allocation by plugin, freed by VFSi)
  //  PMPointerDataType pmfp;         //  -> utility function pointers (currently not supported)
};
typedef struct PMPluginExchangeDataType PMPluginExchangeDataType;
typedef PMPluginExchangeDataType * PMPluginExchangeDataPtr;
// attrbitutes definitions for launch
#define PMEAttrLaunchNormalOpen      0x0001  // launch by filelist item open
#define PMEAttrLaunchByMenu          0x0002  // launch by select menu item
#define PMEAttrLaunchDispNotify      0x0004  // launch by ChangeDispNotify (for silk)
// attributes definitions for quit
#define PMEAttrFilelistDirty         0x0100  // it should be set when you make new file or change file
// attribute definitions for launch and quit (bidirectional)
#define PMEAttrPINState              0x8000  // on-bit means DIA is opend

// Info func datatype definitions
struct PMPluginInfoDataType {
  char *filename;                 //  -> filename (absolute path)                       (Don't be freed)
  EventPtr evp;                   //  -> event ptr
  MemHandle preserveH;            //  <- preserved memory chunk while app is running.   (Don't be freed)
};
typedef struct PMPluginInfoDataType PMPluginInfoDataType;
typedef PMPluginInfoDataType * PMPluginInfoDataPtr;
